
#ifndef __H_BASE_N__
#define __H_BASE_N__

#include "defs.h"

BEGIN_DECLS

/**
 * @file
 * @defgroup BaseN BaseN
 * @addtogroup BaseN
 * @{
 * https://tools.ietf.org/html/rfc4648
 */

/**
 * Encode a Byte array to a Base16 string.
 */
schar *
s_base_16_enc (sbyte * input_data, size_t in_len, size_t * out_len);

/**
 * Decode a base16 encoded string into a Byte array.
 */
sbyte *
s_base_16_dec (schar * base16_str, size_t in_len, size_t * out_len);

/**
 * Encode a Byte array to a Base16 string.
 */
schar *
s_base_32_enc (sbyte * input_data,size_t in_len, size_t * out_len);

/**
 * Decode a base32 encoded string into a Byte array.
 */
sbyte *
s_base_32_dec (schar * base32_str, size_t in_len, size_t * out_len);

/**
 * Encode a Byte array to a Base32 Hex (Extended hex) string.
 */
schar *
s_base_32_hex_enc (sbyte * input_data, size_t in_len, size_t * out_len);

/**
 * Decode a base32 Hex (Extended hex) encoded string into a Byte array.
 */
sbyte *
s_base_32_hex_dec (schar * base32_str, size_t in_len, size_t * out_len);

/**
 * Encode a Byte array to a Base64 string.
 */
schar *
s_base_64_enc (sbyte * input_data, size_t in_len, size_t * out_len);

/**
 * Decode a base64 encoded string into a Byte array.
 */
sbyte *
s_base_64_dec (schar * base64_str, size_t in_len, size_t * out_len);

/**
 * Encode a byte array into a URI safe Base64 string.
 */
schar *
s_base_64_safe_enc (sbyte * input_data, size_t in_len, size_t * out_len);

/**
 * Decode a URI safe Base64 string into a byte array.
 */
sbyte *
s_base_64_safe_dec (schar * base64_str, size_t in_len, size_t * out_len);



/**
 * @}
 */

END_DECLS

#endif __H_BASE_N__
