/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "Error.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "utils.h"

struct SErrorPrivate {
  schar * message;
  SErrorType error_type;
};

schar *
error_to_string_method (SObject * self);
void
error_free_method (SError * self);

SError *
s_error_new (SErrorType error, char * message) {
  SError * self = malloc (sizeof (SError));
  SErrorClass * klass = malloc (sizeof (SErrorClass));
  self->priv = malloc (sizeof (SErrorPrivate));
  
  s_object_set_class (S_OBJECT (self), S_OBJECT_CLASS (klass));
  s_object_initialize (S_OBJECT (self), "Error");
  s_object_set_to_string_method (S_OBJECT (self), error_to_string_method);
  s_object_set_free_method (S_OBJECT (self), METHOD(error_free_method));
  
  self->priv->message = s_string_new (message);
  self->priv->error_type = error;
  return self;
}

void
s_error_free (SError * self) {
  s_object_free (S_OBJECT (self));
}

schar *
error_to_string_method (SObject * obj) {
  SError * self = S_ERROR (obj);
  schar * ret_val = NULL;
  schar * error_type_str = NULL;
  error_type_str = s_string_new (SErrorTypeName[self->priv->error_type]);
  
  ret_val = s_string_new_fmt ("Error: %s, Message: %s", error_type_str, self->priv->message);
  free (error_type_str);
  return ret_val;
}

void
error_free_method (SError * self) {
  free (self->priv->message);
  free (self->priv);
  SErrorClass * klass = S_ERROR_CLASS (s_object_get_class (S_OBJECT (self)));
  free (klass);
  free (self);
}

schar *
s_error_get_name (SErrorType k) {
  return SErrorTypeName[k];
}

