/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/


#ifndef __H_FUNC__
#define __H_FUNC__


#include "defs.h"
#include "utils.h"

BEGIN_DECLS


/** @file
 * Generic function pointer definitions and data stuctures to do with
 * functions.
 * 
 * @todo create an object type that can hold functions and be able to run them.
 */


/*
 * This data structure holds a pointer to a function.
 *
 * A Func object must hold a pointer to a function and can hold a string,
 * representing its name.
 */

#define METHOD(k) ((MethodFunc) k)
#define FUNCTION(k) ((FuncPointer) k)
#define FREEFUNC(k) ((FreeFunc) k)
#define CALLBACK(k) ((Callback) k)
#define FOREACHFUNC(k) ((ForEachFunc) k)

/*
 * Common types of function pointers:
 */


/**
 * Generic function.
 *
 * It can take any type of function, with any signature
 */
typedef void (* FuncPointer)(void);

/**
 * A generic callback function.
 *
 * A callback function can have any signature.
 */
typedef void (* Callback)(void);

/**
 *
 */
typedef void (* FreeFunc)(spointer obj);

typedef spointer (* RunFunc)(spointer user_data);

/**
 * @param obj The object that the function should be run on. Generally this
 *            should be ignored inside the function.
 * @param item The item that the the function should act on. @newline
 *             This can be printing the data, erasing the it, modifying it,
 *             changing it or what ever.
 * @param data The data that is passed to the _foreach () function. Often called
 *             the user data. Can also be used as an out parameter.
 */
typedef void (* ForEachFunc)(spointer obj, spointer item, spointer data);


/**
 * A CompFunc represents a standard comparison function.
 * 
 * @param 1 object to compare
 * @param 2 object to compare
 * 
 * compare parameter 1 with parameter 2.
 */
typedef sboolean (* CompFunc)(void*, void*);

typedef char * (* CharFunc)();


END_DECLS

#endif
