
#ifndef __H_THREAD__
#define __H_THREAD__

#include "defs.h"
#include "utils.h"
#include "Func.h"

BEGIN_DECLS

#include "config.h"

#if  !__STDC_NO_ATOMICS__
#pragma message ("We have _Atomic")
#include <stdatomic.h>
#else
#pragma message ("We don't have standard _Atomic. includeing external .h file.")
#include "external/stdatomic.h"
#endif

#if !__STDC_NO_THREADS__
#pragma message ("We have threads.h")
#include <threads.h>
#else
#pragma message ("We don't have standard threads.h. includeing external .h file.")
#include "external/threads.h"
#endif


/**
 * @defgroup Threading Threading
 * @addtogroup Threading
 * @{
 */


/**
 * Sleep for a set amount of micro seconds.
 */
void
s_usleep (slong us);


/* ****************************************************************************
 ********************************** SMutex ************************************
 **************************************************************************** */

/**
 * @defgroup Mutex Mutex
 * @addtogroup Mutex
 * @{
 */

/**
 * An SMutex is an opaque data type that handles the platform specifics of the
 * Mutex, if it exists. If not we roll our own.
 */
typedef struct SMutex SMutex;

/**
 * Create a new SMutex;
 */
SMutex *
s_mutex_new ();

/**
 * Free the mutex.
 */
void
s_mutex_free (SMutex * self);

/**
 * Lock the mutex.
 *
 * Returns a key used to unlock the mutex.
 */
sint
s_mutex_lock (SMutex * self);

/**
 * unlock the mutex.
 *
 * @param self The mutex to unlock;
 * @param key The key used to unlock the mutex.
 */
sint
s_mutex_unlock (SMutex * self);

/**
 * Check if a mutex is locked.
 *
 * Note that this operation is non-atomic and may be wrong. 
 */
sboolean
s_mutex_check_lock (SMutex * self);

/** @} */

/* ****************************************************************************
 ********************************** SThread ***********************************
 **************************************************************************** */

/**
 * @defgroup SThread SThread
 * @addtogroup SThread
 * @{
 */

/**
 * An opaque data type representing a Thread.
 */
typedef struct SThread SThread;

/**
 * Creat a now Thread.
 */
SThread *
s_thread_new (RunFunc func);

/**
 * Free the thread object.
 */
void
s_thread_free (SThread * self);

/**
 * Run the thread.
 */
sboolean
s_thread_run (SThread * self, spointer user_data);


/** @} */

/** @} */

END_DECLS
#endif /* __H_THREAD__ */
